'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
var VALID_TOP_LEVEL_IMPORT_PATHS = ['styled-components', 'styled-components/no-tags', 'styled-components/native', 'styled-components/primitives'];

var isValidTopLevelImport = exports.isValidTopLevelImport = function isValidTopLevelImport(x) {
    return VALID_TOP_LEVEL_IMPORT_PATHS.includes(x);
};

var importLocalName = function importLocalName(name, state) {
    var localName = name === 'default' ? 'styled' : name;

    state.file.path.traverse({
        ImportDeclaration: {
            exit: function exit(path) {
                var node = path.node;


                if (isValidTopLevelImport(node.source.value)) {
                    var _iteratorNormalCompletion = true;
                    var _didIteratorError = false;
                    var _iteratorError = undefined;

                    try {
                        for (var _iterator = path.get('specifiers')[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                            var specifier = _step.value;

                            if (specifier.isImportDefaultSpecifier()) {
                                localName = specifier.node.local.name;
                            }

                            if (specifier.isImportSpecifier() && specifier.node.imported.name === name) {
                                localName = specifier.node.local.name;
                            }

                            if (specifier.isImportNamespaceSpecifier()) {
                                localName = specifier.node.local.name;
                            }
                        }
                    } catch (err) {
                        _didIteratorError = true;
                        _iteratorError = err;
                    } finally {
                        try {
                            if (!_iteratorNormalCompletion && _iterator.return) {
                                _iterator.return();
                            }
                        } finally {
                            if (_didIteratorError) {
                                throw _iteratorError;
                            }
                        }
                    }
                }
            }
        }
    });

    return localName;
};

var isStyled = exports.isStyled = function isStyled(t) {
    return function (tag, state) {
        /* Matches the extend blocks such as
        const Block = Div.extend`
            color: @color
        `
        */
        if (tag.property && tag.property.name === "extend") {
            return true;
        }

        if (t.isCallExpression(tag) && t.isMemberExpression(tag.callee) && tag.callee.property.name !== 'default' /** ignore default for #93 below */
        ) {
                // styled.something()
                return isStyled(t)(tag.callee.object, state);
            } else {
            return t.isMemberExpression(tag) && tag.object.name === importLocalName('default', state) || t.isCallExpression(tag) && tag.callee.name === importLocalName('default', state) ||
            /**
             * #93 Support require()
             * styled-components might be imported using a require()
             * call and assigned to a variable of any name.
             * - styled.default.div``
             * - styled.default.something()
             */
            state.styledRequired && t.isMemberExpression(tag) && t.isMemberExpression(tag.object) && tag.object.property.name === 'default' && tag.object.object.name === state.styledRequired || state.styledRequired && t.isCallExpression(tag) && t.isMemberExpression(tag.callee) && tag.callee.property.name === 'default' && tag.callee.object.name === state.styledRequired;
        }
    };
};

var isCSSHelper = exports.isCSSHelper = function isCSSHelper(t) {
    return function (tag, state) {
        return t.isIdentifier(tag) && tag.name === importLocalName('css', state);
    };
};

var isCreateGlobalStyleHelper = exports.isCreateGlobalStyleHelper = function isCreateGlobalStyleHelper(t) {
    return function (tag, state) {
        return t.isIdentifier(tag) && tag.name === importLocalName('createGlobalStyle', state);
    };
};

var isInjectGlobalHelper = exports.isInjectGlobalHelper = function isInjectGlobalHelper(t) {
    return function (tag, state) {
        return t.isIdentifier(tag) && tag.name === importLocalName('injectGlobal', state);
    };
};

var isKeyframesHelper = exports.isKeyframesHelper = function isKeyframesHelper(t) {
    return function (tag, state) {
        return t.isIdentifier(tag) && tag.name === importLocalName('keyframes', state);
    };
};

var isHelper = exports.isHelper = function isHelper(t) {
    return function (tag, state) {
        return isCSSHelper(t)(tag, state) || isKeyframesHelper(t)(tag, state);
    };
};