"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _deasync = require("deasync");

var _deasync2 = _interopRequireDefault(_deasync);

var _less = require("less/lib/less/");

var _less2 = _interopRequireDefault(_less);

var _fileManager = require("less/lib/less-node/file-manager");

var _fileManager2 = _interopRequireDefault(_fileManager);

var _utils = require("less/lib/less/utils");

var _utils2 = _interopRequireDefault(_utils);

var _Variable = require("../tree/Variable");

var _Variable2 = _interopRequireDefault(_Variable);

var _Condition = require("../tree/Condition");

var _Condition2 = _interopRequireDefault(_Condition);

var _Negative = require("../tree/Negative");

var _Negative2 = _interopRequireDefault(_Negative);

var _Dimension = require("../tree/Dimension");

var _Dimension2 = _interopRequireDefault(_Dimension);

var _functions = require("../functions");

var functions = _interopRequireWildcard(_functions);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var consumeBrackets = function consumeBrackets(parserInput) {
    var inComment = false;
    var blockDepth = 0;
    var blockStack = [];
    var input = parserInput.getInput();
    var length = input.length;
    var lastPos = parserInput.i;
    var i = parserInput.i;
    var prevChar = void 0,
        nextChar = void 0;

    do {
        prevChar, nextChar = input.charAt(i);
        if (blockDepth === 0 && prevChar === "}") {
            return input.substr(lastPos, i - lastPos);
        } else {
            if (inComment) {
                if (nextChar === '*' && input.charAt(i + 1) === '/') {
                    i++;
                    blockDepth--;
                    inComment = false;
                }
                i++;
                continue;
            }
            switch (nextChar) {
                case '/':
                    if (input.charAt(i + 1) === '*') {
                        i++;
                        inComment = true;
                        blockDepth++;
                    }
                    break;
                case "'":
                case '"':
                    var quote = parserInput.$quoted(i);
                    if (quote) {
                        i += quote[1].length - 1;
                    }
                    break;
                case '{':
                    blockStack.push('}');
                    blockDepth++;
                    break;
                case '(':
                    blockStack.push(')');
                    blockDepth++;
                    break;
                case '[':
                    blockStack.push(']');
                    blockDepth++;
                    break;
                case '}':
                case ')':
                case ']':
                    var expected = blockStack.pop();
                    if (nextChar === expected) {
                        blockDepth--;
                    } else {
                        return expected;
                    }
            }
            i++;
        }
        prevChar = nextChar;
    } while (i <= length);
};

exports.default = function (source, filename) {
    var less = new _less2.default(undefined, [new _fileManager2.default()]);
    var lastSelf = void 0;
    Object.defineProperty(less.tree.Node.prototype, 'parse', {
        get: function get() {
            return lastSelf;
        },
        set: function set(self) {
            lastSelf = self;
            var parseJS = function parseJS(orig, treeConstructor, isDeclaration) {
                var val = orig();
                if (val) return val;

                var name = void 0;
                self.parserInput.save();
                if (self.parserInput.currentChar() === '$' && (name = consumeBrackets(self.parserInput))) {
                    // Makes the assumption that all nested code blocks have css in them.
                    if (isDeclaration && !name.includes("css")) {
                        return;
                    }
                    self.parserInput.$str(name);
                    var anonymous = new less.tree.Anonymous(name, self.parserInput.i, self.fileInfo);
                    anonymous.noSpacing = self.parserInput.prevChar() !== " ";
                    if (treeConstructor) {
                        return treeConstructor(anonymous);
                    } else {
                        return anonymous;
                    }
                }
                self.parserInput.restore();
            };

            self.parsers.declaration = parseJS.bind(null, self.parsers.declaration.bind(self.parsers), undefined, true);
            self.parsers.entities.variable = parseJS.bind(null, self.parsers.entities.variable.bind(self.parsers));
            self.parsers.element = parseJS.bind(null, self.parsers.element.bind(self.parsers), function (anonymous) {
                var c = self.parsers.combinator();
                return new less.tree.Element(c, anonymous, true, self.parserInput.i, self.fileInfo);
            });
        },
        configurable: true
    });
    less.tree.Variable = _Variable2.default;
    less.tree.Condition = _Condition2.default;
    less.tree.Negative = _Negative2.default;
    less.tree.Dimension = _Dimension2.default;
    less.tree.Expression.prototype.genCSS = function (context, output) {
        for (var i = 0; i < this.value.length; i++) {
            this.value[i].genCSS(context, output);
            if (!(this.noSpacing || this.value[i].noSpacing) && i + 1 < this.value.length) {
                output.add(' ');
            }
        }
    };
    less.tree.Anonymous.prototype.eval = function () {
        var anonymous = new less.tree.Anonymous(this.value, this._index, this._fileInfo, this.mapLines, this.rulesetLike, this.visibilityInfo());
        anonymous.noSpacing = this.noSpacing;
        return anonymous;
    };

    // Changes the function joinSelector to allow & selectors in the root element. Useful for overriding styles with higher specificity.
    var Paren = less.tree.Paren;
    var Selector = less.tree.Selector;
    var Element = less.tree.Element;
    var joinSelector = less.tree.Ruleset.prototype.joinSelector.toString().replace("if (el.value !== '&') {", "if (el.value !== '&' || context.length === 0) {");
    eval("less.tree.Ruleset.prototype.joinSelector = " + joinSelector);

    less.functions.functionRegistry.addMultiple(functions);
    less.PluginLoader = function PluginLoader() {
        _classCallCheck(this, PluginLoader);
    };

    var parse = (0, _deasync2.default)(function (input, options, callback) {
        return less.parse(input, options, function (e, root, imports, options) {
            return callback(e, { root: root, imports: imports, options: options });
        });
    });

    var _parse = parse(source, { math: 0, paths: _path2.default.dirname(filename) }),
        root = _parse.root,
        imports = _parse.imports,
        options = _parse.options;

    if (!root) {
        console.error("Failed to parse", source);
        return source;
    }
    // Disables the validation: Properties must be inside selector blocks. They cannot be in the root
    root.firstRoot = false;

    var parseTree = new less.ParseTree(root, imports);

    var _parseTree$toCSS = parseTree.toCSS(options),
        css = _parseTree$toCSS.css;

    return css;
};